# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat 2013-2014
# Author: Lucas Meneghel Rodrigues <lmr@redhat.com>

import logging
import os
import tempfile

from . import lint
from . import reindent
from . import style
from . import inspector
from . import vcs
from . import utils

TMP_FILE_DIR = tempfile.gettempdir()
LOG_FILE_PATH = os.path.join(TMP_FILE_DIR, 'check-patch.log')

log = logging.getLogger("inspektor.check")

# Rely on built-in recursion limit to limit number of directories searched


class FileChecker(object):

    """
    Picks up a given file and performs various checks, looking after problems
    and eventually applying solutions when all possible.
    """

    def __init__(self, args):
        """
        Class constructor, sets the file checkers.

        :param path: Path to the file that will be checked.
        :param vcs: Version control system being used.
        :param confirm: Whether to answer yes to all questions asked without
                prompting the user.
        """
        assert args.disable is not None
        assert args.pep8_disable is not None
        self.args = args
        self.linter = lint.Linter(self.args)
        self.indenter = reindent.Reindenter(self.args)
        # Tweak --disable option for StyleChecker
        self.args.disable = self.args.pep8_disable
        self.style_checker = style.StyleChecker(self.args)
        self.vcs = vcs.VCS()

    def _check_indent(self, path):
        """
        Verifies the file with the reindent module.

        This tool performs the following checks on python files:

          * Trailing whitespaces
          * Tabs
          * End of line
          * Incorrect indentation

        And will automatically correct problems.
        """
        return self.indenter.check_file(path)

    def _check_syntax(self, path):
        """
        Verifies the file with pylint.
        """
        return self.linter.check_file(path)

    def _check_style(self, path):
        """
        Verifies the file compliance to PEP8.
        """
        return self.style_checker.check_file(path)

    def _check_permissions(self, path):
        """
        Verifies the execution permissions and fixes them if possible:

          * Files with no shebang and execution permissions will be fixed.
          * Files with shebang and no execution permissions will be fixed.
        """
        result = True
        if os.path.isdir(path):
            return result
        path_inspector = inspector.PathInspector(path, self.args)
        path_is_script = path_inspector.is_script()
        path_is_exec = path_inspector.has_exec_permission()
        if path_is_script:
            if not path_is_exec:
                result = False
                self.vcs.set_file_executable(path)
        else:
            if path_is_exec:
                result = False
                self.vcs.unset_file_executable(path)
        return result

    def check_file(self, path):
        """
        Executes all required checks, if problems are found, the possible
        corrective actions are listed.
        """
        return (self._check_syntax(path) and
                self._check_indent(path) and
                self._check_style(path) and
                self._check_permissions(path))


class PatchChecker(FileChecker):

    def __init__(self, args, patch=None, patchwork_id=None, github_id=None):
        FileChecker.__init__(self, args)
        self.base_dir = TMP_FILE_DIR

        if patch:
            self.patch = os.path.abspath(patch)

        if patchwork_id:
            self.patch = self._fetch_from_patchwork(patchwork_id)

        if github_id:
            self.patch = self._fetch_from_github(github_id)

    def validate(self):
        if not os.path.isfile(self.patch):
            logging.error("Invalid patch file %s provided. Aborting.",
                          self.patch)
            return 1

        changed_files_before = self.vcs.get_modified_files()
        if changed_files_before:
            log.error("Repository has changed files prior to patch "
                      "application")
            answer = utils.ask("Would you like to revert them?")
            if answer == "n":
                log.error("Not safe to proceed without reverting files.")
                return 1
            else:
                for changed_file in changed_files_before:
                    self.vcs.revert_file(changed_file)

        self.untracked_files = self.vcs.get_unknown_files()

    def _get_patchwork_url(self, pw_id):
        return "http://%s/patch/%s/mbox/" % (self.pwhost, pw_id)

    def _download_from_patchwork(self, pw_id):
        patch_url = self._get_patchwork_url(pw_id)
        patch_dest = os.path.join(self.base_dir, 'patchwork-%s.patch' %
                                  pw_id)
        return utils.download.get_file(patch_url, patch_dest)

    def _fetch_from_patchwork(self, pw_id):
        """
        Gets a patch file from patchwork and puts it under the cwd so it can
        be applied.

        :param pw_id: Patchwork patch id. It can be a string with
                comma separated patchwork ids.
        """
        collection = os.path.join(self.base_dir, 'patchwork-%s.patch' %
                                  utils.random_string(4))
        collection_rw = open(collection, 'w')

        for p_id in pw_id.split(","):
            patch = self._download_from_patchwork(p_id)
            # Patchwork sometimes puts garbage on the path, such as long
            # sequences of underscores (_______). Get rid of those.
            patch_ro = open(patch, 'r')
            patch_contents = patch_ro.readlines()
            patch_ro.close()
            for line in patch_contents:
                if not line.startswith("___"):
                    collection_rw.write(line)
        collection_rw.close()

        return collection

    def _get_github_repo_name(self):
        return self.vcs.get_repo_name()

    def _get_github_url(self, gh_id):
        """
        Gets the correct github URL for the given project.
        """
        p_project = self.args.parent_project
        repo = self._get_github_repo_name()
        return ("https://github.com/%s/%s/pull/%s.patch" %
                (p_project, repo, gh_id))

    def _fetch_from_github(self, gh_id):
        """
        Gets a patch file from patchwork and puts it under the base dir.

        :param gh_id: Patchwork patch id.
        """
        patch_url = self._get_github_url(gh_id)
        patch_dest = os.path.join(self.base_dir, 'github-%s.patch' % gh_id)
        return utils.download.get_file(patch_url, patch_dest)

    def _check_files_modified_patch(self):
        files_failed_check = []

        modified_files = self.vcs.get_modified_files_patch(self.untracked_files,
                                                           self.patch)
        for modified_file in modified_files:
            # Additional safety check, new commits might introduce
            # new directories
            if os.path.isfile(modified_file):
                if not self.check_file(modified_file):
                    files_failed_check.append(modified_file)

        if files_failed_check:
            return False
        else:
            return True

    def check(self):
        self.vcs.apply_patch(self.patch)
        return self._check_files_modified_patch()


def check_patch_github(args):
    gh_id = args.gh_id
    checker = PatchChecker(args, github_id=gh_id)
    checker.validate()
    if checker.check():
        log.info("Github ID #%s check PASS", gh_id)
        return 0
    else:
        log.info("Github ID #%s check FAIL", gh_id)
        return 1


def set_arguments(parser):
    pgh = parser.add_parser('github',
                            help='check GitHub Pull Requests')
    pgh.add_argument('gh_id', type=int,
                     help='GitHub Pull Request ID')
    pgh.add_argument('-p', '--parent-project', type=str,
                     help=('Parent project name of the current repository.'
                           ' Default: %(default)s'),
                     default='autotest')
    pgh.add_argument('--disable', type=str,
                     help='Disable the pylint errors. Default: %(default)s',
                     default='W,R,C,E1002,E1101,E1103,E1120,F0401,I0011')
    pgh.add_argument('--enable', type=str,
                     help=('Enable the pylint errors '
                           '(takes place after disabled items are '
                           'processed). Default: %(default)s'),
                     default='W0611')
    pgh.add_argument('--pep8-disable', type=str,
                     help='Disable the pep8 errors. Default: %(default)s',
                     default='E501,E265,W601,E402')
    return ('check', check_patch_github)



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat 2013-2014
# Author: Lucas Meneghel Rodrigues <lmr@redhat.com>

import logging
import subprocess
import shlex
import time

from inspektor import exceptions

log = logging.getLogger('inspektor.utils')


class CmdResult(object):

    """
    Command execution result.

    command:     String containing the command line itself
    exit_status: Integer exit code of the process
    stdout:      String containing stdout of the process
    stderr:      String containing stderr of the process
    duration:    Elapsed wall clock time running the process
    """

    def __init__(self, command="", stdout="", stderr="",
                 exit_status=None, duration=0):
        self.command = command
        self.exit_status = exit_status
        self.stdout = stdout
        self.stderr = stderr
        self.duration = duration

    def __repr__(self):
        return ("Command: %s\n"
                "Exit status: %s\n"
                "Duration: %s\n"
                "Stdout:\n%s\n"
                "Stderr:\n%s\n" % (self.command, self.exit_status,
                                   self.duration, self.stdout, self.stderr))


def run(cmd, verbose=True, ignore_status=False, shell=False):
    if verbose:
        log.info("Running '%s'", cmd)
    if shell is False:
        args = shlex.split(cmd)
    else:
        args = cmd
    start = time.time()
    p = subprocess.Popen(args,
                         stdout=subprocess.PIPE,
                         stderr=subprocess.PIPE,
                         shell=shell)
    stdout, stderr = p.communicate()
    duration = time.time() - start
    result = CmdResult(cmd)
    result.exit_status = p.returncode
    result.stdout = stdout
    result.stderr = stderr
    result.duration = duration
    if p.returncode != 0 and not ignore_status:
        raise exceptions.CmdError(cmd, result)
    return result



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat 2013-2014
# Author: Lucas Meneghel Rodrigues <lmr@redhat.com>



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.

"""
Traceback standard module plus some additional APIs.
"""
from traceback import format_exception
import logging
import inspect


def tb_info(exc_info):
    """
    Prepare traceback info.

    :param exc_info: Exception info produced by sys.exc_info()
    """
    exc_type, exc_value, exc_traceback = exc_info
    return format_exception(exc_type, exc_value, exc_traceback.tb_next)


def prepare_exc_info(exc_info):
    """
    Prepare traceback info.

    :param exc_info: Exception info produced by sys.exc_info()
    """
    return "".join(tb_info(exc_info))


def log_exc_info(exc_info, logger='root'):
    """
    Log exception info to logger_name.

    :param exc_info: Exception info produced by sys.exc_info()
    :param logger: Name of the logger (defaults to root)
    """
    log = logging.getLogger(logger)
    log.error('')
    called_from = inspect.currentframe().f_back
    log.error("Reproduced traceback from: %s:%s",
              called_from.f_code.co_filename, called_from.f_lineno)
    for line in tb_info(exc_info):
        for l in line.splitlines():
            log.error(l)
    log.error('')


def log_message(message, logger='root'):
    """
    Log message to logger.

    :param message: Message
    :param logger: Name of the logger (defaults to root)
    """
    log = logging.getLogger(logger)
    for line in message.splitlines():
        log.error(line)



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat 2013-2014
# Author: Lucas Meneghel Rodrigues <lmr@redhat.com>

import os
import stat
import fnmatch

PY_EXTENSIONS = ['.py']
SHEBANG = '#!'


class PathInspector(object):

    def __init__(self, path, args):
        self.args = args
        self.path = path
        self.ignore = ['*~', '*#', '*.swp', '*.py?', '*.o']
        if self.args.exclude:
            self.ignore += self.args.exclude.split(',')
        self._read_gitignore()

    def _read_gitignore(self):
        config_ignore = '{home}/.config/git/ignore'.format(
            home=os.environ.get('HOME'))
        git_dir_ignore = '{gitdir}/.config/git/ignore'.format(
            gitdir=os.environ.get('GIT_DIR'))
        dot_ignore = '.gitignore'
        for ign in (dot_ignore, git_dir_ignore, config_ignore):
            if os.path.isfile(ign):
                with open(ign) as f:
                    self.ignore.extend([x.strip() for x in f.readlines()])
                break
        self.ignore = set(self.ignore)

    def get_first_line(self):
        first_line = ""
        if os.path.isfile(self.path):
            checked_file = open(self.path, "r")
            first_line = checked_file.readline()
            checked_file.close()
        return first_line

    def has_exec_permission(self):
        mode = os.stat(self.path)[stat.ST_MODE]
        return mode & stat.S_IXUSR

    def is_empty(self):
        size = os.stat(self.path)[stat.ST_SIZE]
        return size == 0

    def is_script(self, language=None):
        first_line = self.get_first_line()
        if first_line:
            if first_line.startswith(SHEBANG):
                if language is None:
                    return True
                elif language in first_line:
                    return True
        return False

    def is_python(self):
        for extension in PY_EXTENSIONS:
            if self.path.endswith(extension):
                return True

        return self.is_script(language='python')

    def is_toignore(self):
        for pat in self.ignore:
            if fnmatch.fnmatch(self.path, pat):
                return True
            if self.path.startswith(os.path.abspath(pat)):
                return True
        return False



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat 2013-2014
# Author: Lucas Meneghel Rodrigues <lmr@redhat.com>

import os
import tokenize
import logging
import sys

try:
    from os.path import walk
except ImportError:
    from os import walk

from .inspector import PathInspector
from . import stacktrace

log = logging.getLogger("inspektor.reindent")


def _rstrip(line, JUNK='\n \t'):
    """
    Return line stripped of trailing spaces, tabs, newlines.

    Note that line.rstrip() instead also strips sundry control characters,
    but at least one known Emacs user expects to keep junk like that, not
    mentioning Barry by name or anything <wink>.
    """

    i = len(line)
    while i > 0 and line[i - 1] in JUNK:
        i -= 1
    return line[:i]


def _getlspace(line):
    i, n = 0, len(line)
    while i < n and line[i] == " ":
        i += 1
    return i


class Run(object):

    def __init__(self, f):
        self.find_stmt = 1  # next token begins a fresh stmt?
        self.level = 0      # current indent level
        # Raw file lines.
        self.raw = f.readlines()
        f.close()

        # File lines, rstripped & tab-expanded.  Dummy at start is so
        # that we can use tokenize's 1-based line numbering easily.
        # Note that a line is all-blank iff it's "\n".
        self.lines = [_rstrip(line).expandtabs() + "\n"
                      for line in self.raw]
        self.lines.insert(0, None)
        self.index = 1  # index into self.lines of next line

        # List of (lineno, indentlevel) pairs, one for each stmt and
        # comment line.  indentlevel is -1 for comment lines, as a
        # signal that tokenize doesn't know what to do about them;
        # indeed, they're our headache!
        self.stats = []

    def run(self):
        tokenize.tokenize(self.getline, self.tokeneater)
        # Remove trailing empty lines.
        lines = self.lines
        while lines and lines[-1] == "\n":
            lines.pop()
        # Sentinel.
        stats = self.stats
        stats.append((len(lines), 0))
        # Map count of leading spaces to # we want.
        have2want = {}
        # Program after transformation.
        after = self.after = []
        # Copy over initial empty lines -- there's nothing to do until
        # we see a line with *something* on it.
        i = stats[0][0]
        after.extend(lines[1:i])
        for i in range(len(stats) - 1):
            thisstmt, thislevel = stats[i]
            nextstmt = stats[i + 1][0]
            have = _getlspace(lines[thisstmt])
            want = thislevel * 4
            if want < 0:
                # A comment line.
                if have:
                    # An indented comment line.  If we saw the same
                    # indentation before, reuse what it most recently
                    # mapped to.
                    want = have2want.get(have, -1)
                    if want < 0:
                        # Then it probably belongs to the next real stmt.
                        for j in xrange(i + 1, len(stats) - 1):
                            jline, jlevel = stats[j]
                            if jlevel >= 0:
                                if have == _getlspace(lines[jline]):
                                    want = jlevel * 4
                                break
                    if want < 0:
                        for j in xrange(i - 1, -1, -1):
                            jline, jlevel = stats[j]
                            if jlevel >= 0:
                                want = have + _getlspace(after[jline - 1]) - \
                                    _getlspace(lines[jline])
                                break
                    if want < 0:
                        # Still no luck -- leave it alone.
                        want = have
                else:
                    want = 0
            assert want >= 0
            have2want[have] = want
            diff = want - have
            if diff == 0 or have == 0:
                after.extend(lines[thisstmt:nextstmt])
            else:
                for line in lines[thisstmt:nextstmt]:
                    if diff > 0:
                        if line == "\n":
                            after.append(line)
                        else:
                            after.append(" " * diff + line)
                    else:
                        remove = min(_getlspace(line), -diff)
                        after.append(line[remove:])
        return self.raw != self.after

    def write(self, f):
        f.writelines(self.after)

    # Line-getter for tokenize.
    def getline(self):
        if self.index >= len(self.lines):
            line = ""
        else:
            line = self.lines[self.index]
            self.index += 1
        return line

    # Line-eater for tokenize.
    def tokeneater(self, t_type, token, sline_scol, end, line,
                   INDENT=tokenize.INDENT,
                   DEDENT=tokenize.DEDENT,
                   NEWLINE=tokenize.NEWLINE,
                   COMMENT=tokenize.COMMENT,
                   NL=tokenize.NL):

        if t_type == NEWLINE:
            # A program statement, or ENDMARKER, will eventually follow,
            # after some (possibly empty) run of tokens of the form
            #     (NL | COMMENT)* (INDENT | DEDENT+)?
            self.find_stmt = 1

        elif t_type == INDENT:
            self.find_stmt = 1
            self.level += 1

        elif t_type == DEDENT:
            self.find_stmt = 1
            self.level -= 1

        elif t_type == COMMENT:
            if self.find_stmt:
                sline, _ = sline_scol
                self.stats.append((sline, -1))
                # but we're still looking for a new stmt, so leave
                # find_stmt alone

        elif t_type == NL:
            pass

        elif self.find_stmt:
            # This is the first "real token" following a NEWLINE, so it
            # must be the first token of the next program statement, or an
            # ENDMARKER.
            self.find_stmt = 0
            if line:   # not endmarker
                sline, _ = sline_scol
                self.stats.append((sline, self.level))


class Reindenter(object):

    def __init__(self, args):
        self.args = args
        self.failed_paths = []

    def check_file(self, path):
        """
        Check one regular file with pylint for py syntax errors.

        :param path: Path to a regular file.
        :return: False, if reindenter found problems, True, if reindenter
                 didn't find problems, or path is not a python module or
                 script.
        """
        inspector = PathInspector(path=path, args=self.args)
        if inspector.is_toignore():
            return True
        if not inspector.is_python():
            return True
        f = open(path)
        r = Run(f)
        f.close()
        try:
            if r.run():
                log.error('Indentation check fail  : %s', path)
                self.failed_paths.append(path)
                if self.args.fix:
                    f = open(path, "w")
                    r.write(f)
                    f.close()
                    log.info('FIX OK')
                return False
            else:
                return True
        except IndentationError:
            log.error("Indentation check fail  : %s", path)
            log.error("Automated fix impossible: %s", path)
            log.error("Look at the stack trace "
                      "below and fix it manually")
            exc_info = sys.exc_info()
            stacktrace.log_exc_info(exc_info, 'inspektor.reindent')
            return False

    def check_dir(self, path):
        def visit(arg, dirname, filenames):
            for filename in filenames:
                self.check_file(os.path.join(dirname, filename))

        walk(path, visit, None)
        return not self.failed_paths

    def check(self, path):
        if os.path.isfile(path):
            return self.check_file(path)
        elif os.path.isdir(path):
            return self.check_dir(path)
        else:
            log.warning("Invalid location '%s'", path)
            return False


def run_reindent(args):
    paths = args.path
    if not paths:
        paths = [os.getcwd()]

    reindenter = Reindenter(args)

    status = True
    for path in paths:
        status &= reindenter.check(path)
    if status:
        log.info("Indentation check PASS")
        return 0
    else:
        log.error("Indentation check FAIL")
        return 1


def set_arguments(parser):
    command = 'indent'
    pindent = parser.add_parser(command, help='check code indentation')
    pindent.add_argument('path', type=str,
                         help='Path to check (empty for full tree check)',
                         nargs='*',
                         default=None)
    pindent.add_argument('--fix', action='store_true', default=False,
                         help='Fix any indentation problems found')
    return (command, run_reindent)



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat 2013-2014
# Author: Lucas Meneghel Rodrigues <lmr@redhat.com>

import logging
import os
import sys

try:
    from os.path import walk
except ImportError:
    from os import walk

import pycodestyle

try:
    import autopep8
    AUTOPEP8_CAPABLE = True
    del(autopep8)
except ImportError:
    AUTOPEP8_CAPABLE = False

from .inspector import PathInspector
from . import stacktrace
from .utils import process

log = logging.getLogger("inspektor.style")


class StyleChecker(object):

    def __init__(self, args):
        self.args = args
        # Be able to analyze all imports inside the project
        sys.path.insert(0, os.getcwd())
        self.failed_paths = []
        log.info('PEP8 disabled: %s' % self.args.disable)

    def check_dir(self, path):
        """
        Recursively go on a directory checking files with PEP8.

        :param path: Path to a directory.
        """
        def visit(arg, dirname, filenames):
            for filename in filenames:
                self.check_file(os.path.join(dirname, filename))

        walk(path, visit, None)
        return not self.failed_paths

    def check_file(self, path):
        """
        Check one regular file with pylint for py syntax errors.

        :param path: Path to a regular file.
        :return: False, if pylint found syntax problems, True, if pylint didn't
                 find problems, or path is not a python module or script.
        """
        inspector = PathInspector(path=path, args=self.args)
        if inspector.is_toignore():
            return True
        if not inspector.is_python():
            return True
        try:
            opt_obj = pycodestyle.StyleGuide().options
            ignore_list = self.args.disable.split(',') + list(opt_obj.ignore)
            opt_obj.ignore = tuple(set(ignore_list))
            # pylint: disable=E1123
            runner = pycodestyle.Checker(filename=path, options=opt_obj)
            status = runner.check_all()
        except Exception:
            log.error('Unexpected exception while checking %s', path)
            exc_info = sys.exc_info()
            stacktrace.log_exc_info(exc_info, 'inspektor.style')
            status = 1

        if status != 0:
            log.error('Style check fail: %s', path)
            self.failed_paths.append(path)
            if AUTOPEP8_CAPABLE:
                if self.args.fix:
                    log.info('Trying to fix errors with autopep8')
                    try:
                        process.run('autopep8 --in-place --max-line-length=%s --ignore %s %s' % (self.args.max_line_length, self.args.disable, path))
                    except Exception:
                        log.error('Unable to fix errors')
                        exc_info = sys.exc_info()
                        stacktrace.log_exc_info(exc_info, 'inspektor.style')

        return status == 0

    def check(self, path):
        if os.path.isfile(path):
            return self.check_file(path)
        elif os.path.isdir(path):
            return self.check_dir(path)
        else:
            log.error("Invalid location '%s'", path)
            return False


def run_style(args):
    paths = args.path
    if not paths:
        paths = [os.getcwd()]

    style_checker = StyleChecker(args)

    status = True
    for path in paths:
        status &= style_checker.check(path)
    if status:
        log.info("PEP8 compliance check PASS")
        return 0
    else:
        log.error("PEP8 compliance FAIL")
        return 1


def set_arguments(parser):
    command = 'style'
    pstyle = parser.add_parser(command,
                               help='check code compliance to PEP8')
    pstyle.add_argument('path', type=str,
                        help='Path to check (empty for full tree check)',
                        nargs='*',
                        default=None)
    pstyle.add_argument('--disable', type=str,
                        help='Disable the pep8 errors. Default: %(default)s',
                        default='E501,E265,W601,E402')
    pstyle.add_argument('--fix', action='store_true', default=False,
                        help='Fix any style problems found (with autopep8)')
    pstyle.add_argument('--max-line-length', type=int, default=79,
                        help=('set maximum allowed line length. Default: '
                              '%(default)s'))
    return (command, run_style)



#!/usr/bin/env python

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat 2013-2014
# Author: Lucas Meneghel Rodrigues <lmr@redhat.com>

from __future__ import print_function

__all__ = ['MAJOR', 'MINOR', 'RELEASE', 'VERSION']


MAJOR = 0
MINOR = 3
RELEASE = 0

VERSION = "%s.%s.%s" % (MAJOR, MINOR, RELEASE)

if __name__ == '__main__':
    print(VERSION)



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat 2013-2014
# Author: Lucas Meneghel Rodrigues <lmr@redhat.com>

import logging.config

DEFAULT_LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'brief': {
            'format': '%(message)s',
        },
    },
    'handlers': {
        'console': {
            'level': 'INFO',
            'class': 'logging.StreamHandler',
            'formatter': 'brief',
        },
    },
    'loggers': {
        'inspektor': {
            'handlers': ['console'],
        },
        'inspektor.app': {
            'handlers': ['console'],
            'level': 'INFO',
            'propagate': False,
        },
        'inspektor.lint': {
            'handlers': ['console'],
            'level': 'INFO',
            'propagate': False,
        },
        'inspektor.license': {
            'handlers': ['console'],
            'level': 'INFO',
            'propagate': False,
        },
        'inspektor.style': {
            'handlers': ['console'],
            'level': 'INFO',
            'propagate': False,
        },
        'inspektor.utils': {
            'handlers': ['console'],
            'level': 'INFO',
            'propagate': False,
        },
        'inspektor.check': {
            'handlers': ['console'],
            'level': 'INFO',
            'propagate': False,
        },
        'inspektor.reindent': {
            'handlers': ['console'],
            'level': 'INFO',
            'propagate': False,
        },
        'inspektor.vcs': {
            'handlers': ['console'],
            'level': 'INFO',
            'propagate': False,
        },
    }
}


from logging import config

if not hasattr(config, 'dictConfig'):
    from logutils import dictconfig
    cfg_func = dictconfig.dictConfig
else:
    cfg_func = config.dictConfig

cfg_func(DEFAULT_LOGGING)



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat 2013-2014
# Author: Lucas Meneghel Rodrigues <lmr@redhat.com>

import logging
import os
import socket
import shutil
import urllib2
import urlparse

log = logging.getLogger('inspektor.utils')


def is_url(path):
    """
    Return true if path looks like a URL
    """
    url_parts = urlparse.urlparse(path)
    return (url_parts[0] in ('http', 'https', 'ftp', 'git'))


def url_open(url, data=None, timeout=5):
    """
    Wrapper to urllib2.urlopen with timeout addition.
    """
    # Save old timeout
    old_timeout = socket.getdefaulttimeout()
    socket.setdefaulttimeout(timeout)
    try:
        return urllib2.urlopen(url, data=data)
    finally:
        socket.setdefaulttimeout(old_timeout)


def url_download(url, filename, data=None, timeout=300):
    """
    Retrieve a file from given url.
    """
    log.info('Fetching %s -> %s', url, filename)

    src_file = url_open(url, data=data, timeout=timeout)
    try:
        dest_file = open(filename, 'wb')
        try:
            shutil.copyfileobj(src_file, dest_file)
        finally:
            dest_file.close()
    finally:
        src_file.close()


def get_file(src, dest, permissions=None):
    """
    Get a file from src, which can be local or a remote URL
    """
    if src == dest:
        return

    if is_url(src):
        url_download(src, dest)
    else:
        shutil.copyfile(src, dest)

    if permissions:
        os.chmod(dest, permissions)
    return dest



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat 2013-2014
# Author: Lucas Meneghel Rodrigues <lmr@redhat.com>

"""
Implements the base inspektor application.
"""
import logging
from argparse import ArgumentParser

from .. import lint
from .. import reindent
from .. import style
from .. import check
from .. import license

log = logging.getLogger("inspektor.app")

ERROR_INVALID_ARGS = 1
ERROR_INTERRUPTED = 3


class InspektorApp(object):

    """
    Basic inspektor application.
    """

    def __init__(self):
        self.actions = {}
        self.arg_parser = ArgumentParser(description='Inspektor code check')
        self.arg_parser.add_argument('-v', '--verbose', action='store_true',
                                     help=('print extra debug messages '
                                           '(alters behavior of the lint '
                                           'subcommand)'),
                                     dest='verbose')
        self.arg_parser.add_argument('--exclude', type=str,
                                     help='Quoted string containing paths or '
                                          'patterns to be excluded from '
                                          'checking, comma separated')
        subparsers = self.arg_parser.add_subparsers(title='subcommands',
                                                    description='valid subcommands',
                                                    help='subcommand help',
                                                    dest='subcommand')
        for mod in (lint, reindent, style, check, license):
            command, func = mod.set_arguments(subparsers)
            self.actions[command] = func
        self.args = self.arg_parser.parse_args()

    def run(self):
        try:
            if self.args.subcommand is None:
                self.arg_parser.print_usage()
                return ERROR_INVALID_ARGS
            else:
                return self.actions[self.args.subcommand](self.args)
        except KeyboardInterrupt:
            log.error('User pressed Ctrl+C, exiting...')
            return ERROR_INTERRUPTED



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat 2013-2014
# Author: Lucas Meneghel Rodrigues <lmr@redhat.com>

# pylint: disable=E0611
from setuptools import setup
import sys

if sys.version_info[:2] == (2, 6):
    REQUIRES = ['astroid==1.2.1', 'pycodestyle>=2.0.0', 'pylint==1.3.1', 'logutils>=0.3.3']
elif sys.version_info[:2] == (2, 7):
    REQUIRES = ['pycodestyle>=2.0.0', 'pylint>=1.3']
elif sys.version_info[0] == 3:
    REQUIRES = ['pycodestyle', 'pylint']


setup(name='inspektor',
      version='0.3.0',
      description='Inspektor code checker',
      author='Lucas Meneghel Rodrigues',
      author_email='lookkas@gmail.com',
      url='https://github.com/avocado-framework/inspektor',
      packages=['inspektor',
                'inspektor.cli',
                'inspektor.utils'
                ],
      install_requires=REQUIRES,
      scripts=['scripts/inspekt'],
      )
